#ifndef DISK_FILE_HPP
#define DISK_FILE_HPP

#include <string>
#include <wchar.h>
#include "file.hpp"

namespace t3d{

class DiskFile : public File
{
public:
    DiskFile(const std::wstring &fileName, FileOpenMode fileOpenMode)
        : mfileLen(0)
    {
        m_file = fopen(std::string(fileName.begin(), fileName.end()).c_str(), fileOpenMode == FileOpenMode::Read ? "rb" : "wb");
        seek(0, OffsetCursor::End);
        mfileLen = tell();
        seek(0, OffsetCursor::Set);
    }

    virtual int read(void *data, uint64_t size)
    {
        return fread(data, 1, size, m_file);
    }

    virtual int write(void *data, uint64_t size)
    {
        return fwrite(data, 1, size, m_file);
    }

    virtual void seek(uint64_t offset, OffsetCursor offsetCursor)
    {
        int cur = SEEK_CUR;
        switch (offsetCursor){
        case OffsetCursor::Set:
            cur = SEEK_SET;
            break;
        case OffsetCursor::End:
            cur = SEEK_END;
            break;
        default:
            cur = SEEK_CUR;
        }

        fseek(m_file, offset, cur);
    }

    virtual uint64_t tell() const
    {
        return ftell(m_file);
    }

    virtual void close()
    {
        if (m_file)
            fclose(m_file);
        m_file = 0;
    }

    virtual bool isEof() const
    {
        return mfileLen <= tell();
    }

    virtual ~DiskFile()
    {
        close();
    }

private:
    FILE *m_file;
    uint64_t mfileLen;
};

}

#endif
